/** An attribute definition within a table scheme definition */
class SchemeDefAttribute {
    constructor(
        name,       // String indicating attribute name
        datatype,   // String indicating data type (or undefined)
        isPrimary,  // Optional: True if this attribute is (part of) primary key
        reference   // Optional: SchemeDefReference instance describing foreign key constraint
    ) {
        this.name = name;
        if (datatype)
            this.datatype = datatype;
        if (isPrimary)
            this.isPrimary = true;
        if (reference instanceof SchemeDefReference)
            this.reference = reference;
    }

    /**
    * Check if datatype declaration fits requirements
    * @return mixed         Not falsey string describing first error or `false` (meaning no error)
    */
    validateDatatype(
        arrAllowedDatatypes     // Array of strings representing all allowed data types
    ) {
        if (!(arrAllowedDatatypes instanceof Array) && this.datatype === undefined)
            return false;
        if (!(arrAllowedDatatypes instanceof Array))
            return 'Bei dieser Aufgabe soll darauf verzichtet werden, Datentypen anzugeben.';
        if (arrAllowedDatatypes.indexOf(this.datatype) != -1)
            return false;
        if (this.datatype === undefined)
            return 'Bei dieser Aufgabe müssen Datentypen von Attributen angegeben werden.';
        return '„' + this.datatype + '“ ist kein erlaubter Datentyp.';
    }

    /**
    * Check if attribute definition is semanticaly correct
    * @return mixed         Not falsey string describing first error or `false` (meaning no error)
    */
    validate(
        table,                  // SchemeDefTable instance this instance is part of
        defs,                   // SchemeDefs instance containing this object
        arrAllowedDatatypes     // Array of strings representing all allowed data types
    ) {
        return this.validateDatatype(arrAllowedDatatypes) || (this.reference ? this.reference.validate(defs, this, table.table) : false);
    }
}

/** String indicating attribute name */
SchemeDefAttribute.prototype.name = '?';

/** String indicating data type (or undefined) */
SchemeDefAttribute.prototype.datatype = undefined;

/** Is this attribute (part of) primary key? */
SchemeDefAttribute.prototype.isPrimary = false;

/** SchemeDefReference instance describing foreign key constraint or null */
SchemeDefAttribute.prototype.reference = null;
